
/*******************************************************************                   
                 2-D INVERSE REDUNDANT WAVELET TRANSFORM

calculated with quadratic spline, according to Mallat and Zhong.
It is a redundant (nondownsampled) transform, which produces a low pass
image and TWO detail images at each resolution scale.

Author: Aleksandra Pizurica TELIN/RUG 1998.

********************************************************************/ 
   
  
#include "mex.h"
#include <math.h>

void wt1_mz(double *a, double *h, double *c, int p, int N, int D);


void workFcn(double *Y, double **A, double **DX, double **DY, int Scale, int M, int N)
{  
 
int i,j;
double *Row_In, *Row_Out, *Column_In, *Column_Out;
double k_Filt[7]={0,-0.0078125,-0.054685,-0.171875,0.171875,0.054685,0.0078125};
double l_Filt[7]={0.0078125,0.046875,0.1171875,0.65625,0.1171875,0.046875,0.0078125};
double hi_Filt[7]={0,0,0.125,0.375,0.375,0.125,0};
double k_Filt_Length=7, l_Filt_Length=7, hi_Filt_Length=7;

Row_In=malloc(N*sizeof(double)); 
Column_In=malloc(M*sizeof(double));
Row_Out=malloc(N*sizeof(double)); 
Column_Out=malloc(M*sizeof(double));  


for (i=0; i<M; i++){
   for(j=0; j<N; j++)
     Row_In[j]=DX[i][j];
   wt1_mz(Row_In, k_Filt, Row_Out, Scale, N, k_Filt_Length);
   for(j=0; j<N; j++)
      DX[i][j]=Row_Out[j];

   for(j=0; j<N; j++)
     Row_In[j]=DY[i][j];
   wt1_mz(Row_In, l_Filt, Row_Out, Scale, N, l_Filt_Length);
   for(j=0; j<N; j++)
      DY[i][j]=Row_Out[j];

   for(j=0; j<N; j++)
     Row_In[j]=A[i][j];
   wt1_mz(Row_In, hi_Filt, Row_Out, Scale, N, hi_Filt_Length);
   for(j=0; j<N; j++)
      A[i][j]=Row_Out[j];
}

for (j=0; j<N; j++){
   for(i=0; i<M; i++)
     Column_In[i]=DX[i][j];
   wt1_mz(Column_In, l_Filt, Column_Out, Scale, M, l_Filt_Length);
   for(i=0; i<M; i++)
      DX[i][j]=Column_Out[i];

   for(i=0; i<M; i++)
     Column_In[i]=DY[i][j];
   wt1_mz(Column_In, k_Filt, Column_Out, Scale, M, k_Filt_Length);
   for(i=0; i<M; i++)
      DY[i][j]=Column_Out[i];

   for(i=0; i<M; i++)
     Column_In[i]=A[i][j];
   wt1_mz(Column_In, hi_Filt, Column_Out, Scale, M, hi_Filt_Length);
   for(i=0; i<M; i++)
      A[i][j]=Column_Out[i];
}
    
for(i=0; i<M; i++)
  for(j=0; j<N; j++)
    Y[i+j*M]=A[i][j]+DX[i][j]+DY[i][j]; 

free(Row_In);
free(Row_Out);
free(Column_In);
free(Column_Out);

} /* End of workFcn */


/*---------------------------------------------------------------------------------*/

void wt1_mz(double *a, double *h, double *c, int p, int N, int D)
{  
int i,j,Dext,I1,I2,len,t;
double *ap,*he;

t=pow(2,p);

ap=malloc(3*N*sizeof(double));
Dext=(D-1)*t+1;
he=malloc(Dext*sizeof(double));

/* Solving the border problems: Input vector should be made periodical and mirrored*/

 for (i=0; i<N; i++)
   {
     ap[i+N]=a[i];
     ap[3*N-1-i]=a[i];
     ap[N-1-i]=a[i];
   }

/* Extending the impulse response h: Putting 2^p-1 zeros between the coefficients of h*/

 for(i=0; i<Dext; i++)
    he[i]=0;

 for(i=0; i<D; i++)
    he[i*t]=h[i];

/* Convolution: c=conv(ap,he) */

 I2=((D-1)/2)*t;  /*index of the element h(0) in the vector he*/

 for(j=0; j<N; j++){
   c[j]=0;
   I1=N+j;          /*index of the element a(0+j) in the vector ap*/
   if ((I1+I2+1)>=Dext)
      len=Dext;
   else
      len=I1+I2+1;
   for(i=0; i<len; i++){
      c[j]+=he[i]*ap[I1+I2-i];
   }
 }
   

free(ap);
free(he);
       
}  /* End of wt1_mz*/


/*----------------------------------------------------------------------------------*/

/*Defining the gateway function, i.e. the mex function*/

#define LOWPASS prhs[0]
#define HOR_DETAIL prhs[1]
#define VER_DETAIL prhs[2]
#define SCALE prhs[3]
#define ARRAY_OUT plhs[0]


void mexFunction(
               int nlhs,
               mxArray  *plhs[],
               int     nrhs,
               const mxArray  *prhs[]
               )
{
  int row, col, i, M, N;
  int Scale;
  double *ArrayOut, **Lowpass, **Hor_Detail, **Ver_Detail;

  if (nlhs!=1)
      mexErrMsgTxt("iwt2_mz requires one  output argument [ARRAY_OUT].");
  if (nrhs!=4)
      mexErrMsgTxt("iwt2_mz requires four input arguments [LOWPASS HOR_DETAIL VER_DETAIL SCALE]");
  

  M=mxGetM(LOWPASS);
  N=mxGetN(LOWPASS);

  Scale=mxGetScalar(SCALE); 
  
  /* Allocate memory for return matrices */
  ARRAY_OUT = mxCreateDoubleMatrix(M, N, mxREAL);  
  ArrayOut = mxGetPr(ARRAY_OUT);
  
 
  /* Dynamic allocation of memory for input matrices*/

  Lowpass = malloc(M*sizeof(int));
  for(i=0;i<M;i++)
    Lowpass[i]=malloc(N*sizeof(double));

  Hor_Detail = malloc(M*sizeof(int));
  for(i=0;i<M;i++)
    Hor_Detail[i]=malloc(N*sizeof(double));

  Ver_Detail = malloc(M*sizeof(int));
  for(i=0;i<M;i++)
    Ver_Detail[i]=malloc(N*sizeof(double));


  /* Convert ARRAY_IN to a 2x2 C array (MATLAB stores a two-dimensional matrix 
     in memory as a one-dimensional array) */

  for (col=0; col<N; col++){
    for (row=0; row<M; row++){
        Lowpass[row][col] = (mxGetPr(LOWPASS))[row+col*M];
        Hor_Detail[row][col] = (mxGetPr(HOR_DETAIL))[row+col*M];
        Ver_Detail[row][col] = (mxGetPr(VER_DETAIL))[row+col*M];
    }
  }


/*Call workFcn function*/ 
workFcn(ArrayOut,Lowpass,Hor_Detail,Ver_Detail,Scale,M,N);

for(i=0;i<M;i++)
 free(Lowpass[i]);
free(Lowpass);

for(i=0;i<M;i++)
 free(Hor_Detail[i]);
free(Hor_Detail);

for(i=0;i<M;i++)
 free(Ver_Detail[i]);
free(Ver_Detail);
 
}/*mexFcn*/





