%Demo program wdecomp_1d.m
%
%Generates a noisy signal and calculated and displays its
%non-decimated wavelet transform using the quadratic spline wavelet
%of Mallat&Zhong'92. 


%WNOISE Generate noisy wavelet test data.
%X = WNOISE(NUM,N) returns values of test function 
%number NUM, on a 2^N sample of [0,1].
%[X,XN] = WNOISE(NUM,N,SNRAT) returns previous vector X
%rescaled such that std(x) = SNRAT, 
%and in vector XN the values of vector X corrupted by an
%additive Gaussian white noise N(0,1), then XN has a
%      signal-to-noise ratio of SNRAT.

%The six fonctions are due to Donoho, Johnstone in
%"Adapting to unknown smoothness via wavelet shrinkage"
%Preprint Stanford, january 93, p 27-28.
%NUM = 1         Blocks
%NUM = 2         Bumps
%NUM = 3         Heavy sine
%NUM = 4         Doppler
%NUM = 5         Quadchirp
%NUM = 6         Mishmash

N=6;
NUM=input('Choose test signal: \n1 - Blocks\n2 - Bumps\n3 - Heavy sine\n4 - Doppler\n5 - Quadchirp\n5 - Mishmash\n >> ');
SNRAT=input('Signal to noise ratio: ');
[x,xn] = wnoise(NUM,N,SNRAT);


%Decomposition
  
  [a1,d1]=wt1_mz(xn,0);   
  [a2,d2]=wt1_mz(a1,1);   
  [a3,d3]=wt1_mz(a2,2);   
  [a4,d4]=wt1_mz(a3,3);   


%Reconstruction

   a2r=iwt1_mz(a3,d3,2);
   a1r=iwt1_mz(a2r,d2,1);
   a0r=iwt1_mz(a1r,d1,0);
   
%Compensation for accurate local Lipschitz estimation 
%(duet to Mallat&Zhong'92):

  d1=d1/1.5;
  d2=d2/1.12;
  d3=d3/1.03;
  d4=d4/1.01;
  
  
  % Display the wavelet decomposition
   figure('units','normalized', 'position',[0.1 0.1 0.4 0.8]);
   subplot(5,1,1),plot(xn) ;ylabel('input signal');
   subplot(5,1,2),plot(d1); ylabel('scale 2^1');
   subplot(5,1,3),plot(d2); ylabel('scale 2^2');
   subplot(5,1,4),plot(d3); ylabel('scale 2^3');
   subplot(5,1,5),plot(d4); ylabel('scale 2^4');
   
% Compensating for shifts
   d1_unshift=UnshiftMZwavelet(d1,1,1);
   d2_unshift=UnshiftMZwavelet(d2,2,1);
   d3_unshift=UnshiftMZwavelet(d3,4,1);
   d4_unshift=UnshiftMZwavelet(d4,8,1);

% Display the wavelet decomposition
   figure('units','normalized', 'position',[0.1 0.1 0.4 0.8]);
   subplot(5,1,1),plot(xn) ;ylabel('input signal');
   subplot(5,1,2),plot(d1_unshift); ylabel('scale 2^1');
   subplot(5,1,3),plot(d2_unshift); ylabel('scale 2^2');
   subplot(5,1,4),plot(d3_unshift); ylabel('scale 2^3');
   subplot(5,1,5),plot(d4_unshift); ylabel('scale 2^4');
   
 